@@ -12,30 +12,60 @@
 # The above copyright notice and this permission notice shall be
 # included in all copies or substantial portions of the Software.
 
-"""Representation of type information for DBCore model fields.
-"""
+"""Representation of type information for DBCore model fields."""
+
+from __future__ import annotations
+
+import typing
+from abc import ABC
+from typing import TYPE_CHECKING, Any, Generic, TypeVar, cast
 
-from . import query
 from beets.util import str2bool
 
+from .query import (
+    BooleanQuery,
+    FieldQueryType,
+    NumericQuery,
+    SQLiteType,
+    SubstringQuery,
+)
 
-# Abstract base.
 
-class Type:
+class ModelType(typing.Protocol):
+    """Protocol that specifies the required constructor for model types,
+    i.e. a function that takes any argument and attempts to parse it to the
+    given type.
+    """
+
+    def __init__(self, value: Any = None): ...
+
+
+# Generic type variables, used for the value type T and null type N (if
+# nullable, else T and N are set to the same type for the concrete subclasses
+# of Type).
+if TYPE_CHECKING:
+    N = TypeVar("N", default=Any)
+    T = TypeVar("T", bound=ModelType, default=Any)
+else:
+    N = TypeVar("N")
+    T = TypeVar("T", bound=ModelType)
+
+
+class Type(ABC, Generic[T, N]):
     """An object encapsulating the type of a model field. Includes
     information about how to store, query, format, and parse a given
     field.
     """
 
-    sql = 'TEXT'
+    sql: str = "TEXT"
     """The SQLite column type for the value.
     """
 
-    query = query.SubstringQuery
+    query: FieldQueryType = SubstringQuery
     """The `Query` subclass to be used when querying the field.
     """
 
-    model_type = str
+    model_type: type[T]
     """The Python type that is used to represent the value in the model.
 
     The model is guaranteed to return a value of this type if the field
@@ -44,12 +74,14 @@ class Type:
     """
 
     @property
-    def null(self):
-        """The value to be exposed when the underlying value is None.
-        """
-        return self.model_type()
+    def null(self) -> N:
+        """The value to be exposed when the underlying value is None."""
+        # Note that this default implementation only makes sense for T = N.
+        # It would be better to implement `null()` only in subclasses, or
+        # have a field null_type similar to `model_type` and use that here.
+        return cast(N, self.model_type())
 
-    def format(self, value):
+    def format(self, value: N | T) -> str:
         """Given a value of this type, produce a Unicode string
         representing the value. This is used in template evaluation.
         """
@@ -57,13 +89,13 @@ class Type:
             value = self.null
         # `self.null` might be `None`
         if value is None:
-            value = ''
-        if isinstance(value, bytes):
-            value = value.decode('utf-8', 'ignore')
+            return ""
+        elif isinstance(value, bytes):
+            return value.decode("utf-8", "ignore")
+        else:
+            return str(value)
 
-        return str(value)
-
-    def parse(self, string):
+    def parse(self, string: str) -> T | N:
         """Parse a (possibly human-written) string and return the
         indicated value of this type.
         """
@@ -72,19 +104,20 @@ class Type:
         except ValueError:
             return self.null
 
-    def normalize(self, value):
+    def normalize(self, value: Any) -> T | N:
         """Given a value that will be assigned into a field of this
         type, normalize the value to have the appropriate type. This
         base implementation only reinterprets `None`.
         """
+        # TYPING ERROR
         if value is None:
             return self.null
         else:
             # TODO This should eventually be replaced by
             # `self.model_type(value)`
-            return value
+            return cast(T, value)
 
-    def from_sql(self, sql_value):
+    def from_sql(self, sql_value: SQLiteType) -> T | N:
         """Receives the value stored in the SQL backend and return the
         value to be stored in the model.
 
@@ -99,13 +132,13 @@ class Type:
         and the method must handle these in addition.
         """
         if isinstance(sql_value, memoryview):
-            sql_value = bytes(sql_value).decode('utf-8', 'ignore')
+            sql_value = bytes(sql_value).decode("utf-8", "ignore")
         if isinstance(sql_value, str):
             return self.parse(sql_value)
         else:
             return self.normalize(sql_value)
 
-    def to_sql(self, model_value):
+    def to_sql(self, model_value: Any) -> SQLiteType:
         """Convert a value as stored in the model object to a value used
         by the database adapter.
         """
@@ -114,18 +147,23 @@ class Type:
 
 # Reusable types.
 
-class Default(Type):
-    null = None
+
+class Default(Type[str, None]):
+    model_type = str
+
+    @property
+    def null(self):
+        return None
 
 
-class Integer(Type):
-    """A basic integer type.
-    """
-    sql = 'INTEGER'
-    query = query.NumericQuery
+class BaseInteger(Type[int, N]):
+    """A basic integer type."""
+
+    sql = "INTEGER"
+    query = NumericQuery
     model_type = int
 
-    def normalize(self, value):
+    def normalize(self, value: Any) -> int | N:
         try:
             return self.model_type(round(float(value)))
         except ValueError:
@@ -134,91 +172,153 @@ class Integer(Type):
             return self.null
 
 
-class PaddedInt(Integer):
+class Integer(BaseInteger[int]):
+    @property
+    def null(self) -> int:
+        return 0
+
+
+class NullInteger(BaseInteger[None]):
+    @property
+    def null(self) -> None:
+        return None
+
+
+class BasePaddedInt(BaseInteger[N]):
     """An integer field that is formatted with a given number of digits,
     padded with zeroes.
     """
-    def __init__(self, digits):
+
+    def __init__(self, digits: int):
         self.digits = digits
 
-    def format(self, value):
-        return '{0:0{1}d}'.format(value or 0, self.digits)
+    def format(self, value: int | N) -> str:
+        return "{0:0{1}d}".format(value or 0, self.digits)
 
 
-class NullPaddedInt(PaddedInt):
-    """Same as `PaddedInt`, but does not normalize `None` to `0.0`.
-    """
-    null = None
+class PaddedInt(BasePaddedInt[int]):
+    pass
+
+
+class NullPaddedInt(BasePaddedInt[None]):
+    """Same as `PaddedInt`, but does not normalize `None` to `0`."""
+
+    @property
+    def null(self) -> None:
+        return None
 
 
 class ScaledInt(Integer):
     """An integer whose formatting operation scales the number by a
     constant and adds a suffix. Good for units with large magnitudes.
     """
-    def __init__(self, unit, suffix=''):
+
+    def __init__(self, unit: int, suffix: str = ""):
         self.unit = unit
         self.suffix = suffix
 
-    def format(self, value):
-        return '{}{}'.format((value or 0) // self.unit, self.suffix)
+    def format(self, value: int) -> str:
+        return "{}{}".format((value or 0) // self.unit, self.suffix)
 
 
-class Id(Integer):
+class Id(NullInteger):
     """An integer used as the row id or a foreign key in a SQLite table.
     This type is nullable: None values are not translated to zero.
     """
-    null = None
 
-    def __init__(self, primary=True):
+    @property
+    def null(self) -> None:
+        return None
+
+    def __init__(self, primary: bool = True):
         if primary:
-            self.sql = 'INTEGER PRIMARY KEY'
+            self.sql = "INTEGER PRIMARY KEY"
 
 
-class Float(Type):
+class BaseFloat(Type[float, N]):
     """A basic floating-point type. The `digits` parameter specifies how
     many decimal places to use in the human-readable representation.
     """
-    sql = 'REAL'
-    query = query.NumericQuery
+
+    sql = "REAL"
+    query: FieldQueryType = NumericQuery
     model_type = float
 
-    def __init__(self, digits=1):
+    def __init__(self, digits: int = 1):
         self.digits = digits
 
-    def format(self, value):
-        return '{0:.{1}f}'.format(value or 0, self.digits)
+    def format(self, value: float | N) -> str:
+        return "{0:.{1}f}".format(value or 0, self.digits)
 
 
-class NullFloat(Float):
-    """Same as `Float`, but does not normalize `None` to `0.0`.
-    """
-    null = None
+class Float(BaseFloat[float]):
+    """Floating-point type that normalizes `None` to `0.0`."""
+
+    @property
+    def null(self) -> float:
+        return 0.0
 
 
-class String(Type):
-    """A Unicode string type.
-    """
-    sql = 'TEXT'
-    query = query.SubstringQuery
+class NullFloat(BaseFloat[None]):
+    """Same as `Float`, but does not normalize `None` to `0.0`."""
 
-    def normalize(self, value):
+    @property
+    def null(self) -> None:
+        return None
+
+
+class BaseString(Type[T, N]):
+    """A Unicode string type."""
+
+    sql = "TEXT"
+    query = SubstringQuery
+
+    def normalize(self, value: Any) -> T | N:
         if value is None:
             return self.null
         else:
             return self.model_type(value)
 
 
-class Boolean(Type):
-    """A boolean type.
+class String(BaseString[str, Any]):
+    """A Unicode string type."""
+
+    model_type = str
+
+
+class DelimitedString(BaseString[list[str], list[str]]):
+    """A list of Unicode strings, represented in-database by a single string
+    containing delimiter-separated values.
     """
-    sql = 'INTEGER'
-    query = query.BooleanQuery
+
+    model_type = list
+
+    def __init__(self, delimiter: str):
+        self.delimiter = delimiter
+
+    def format(self, value: list[str]):
+        return self.delimiter.join(value)
+
+    def parse(self, string: str):
+        if not string:
+            return []
+        return string.split(self.delimiter)
+
+    def to_sql(self, model_value: list[str]):
+        return self.delimiter.join(model_value)
+
+
+class Boolean(Type):
+    """A boolean type."""
+
+    sql = "INTEGER"
+    query = BooleanQuery
     model_type = bool
 
-    def format(self, value):
+    def format(self, value: bool) -> str:
         return str(bool(value))
 
-    def parse(self, string):
+    def parse(self, string: str) -> bool:
         return str2bool(string)
 
 
@@ -231,3 +331,7 @@ FLOAT = Float()
 NULL_FLOAT = NullFloat()
 STRING = String()
 BOOLEAN = Boolean()
+SEMICOLON_SPACE_DSV = DelimitedString(delimiter="; ")
+
+# Will set the proper null char in mediafile
+MULTI_VALUE_DSV = DelimitedString(delimiter="\\␀")
