@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # This file is part of beets.
 # Copyright 2016, Thomas Scholtes.
 #
@@ -13,35 +12,33 @@
 # The above copyright notice and this permission notice shall be
 # included in all copies or substantial portions of the Software.
 
-"""Uses the `KeyFinder` program to add the `initial_key` field.
-"""
-
-from __future__ import division, absolute_import, print_function
+"""Uses the `KeyFinder` program to add the `initial_key` field."""
 
 import os.path
 import subprocess
 
-from beets import ui
-from beets import util
+from beets import ui, util
 from beets.plugins import BeetsPlugin
 
 
 class KeyFinderPlugin(BeetsPlugin):
-
     def __init__(self):
-        super(KeyFinderPlugin, self).__init__()
-        self.config.add({
-            u'bin': u'KeyFinder',
-            u'auto': True,
-            u'overwrite': False,
-        })
+        super().__init__()
+        self.config.add(
+            {
+                "bin": "KeyFinder",
+                "auto": True,
+                "overwrite": False,
+            }
+        )
 
-        if self.config['auto'].get(bool):
+        if self.config["auto"].get(bool):
             self.import_stages = [self.imported]
 
     def commands(self):
-        cmd = ui.Subcommand('keyfinder',
-                            help=u'detect and add initial key from audio')
+        cmd = ui.Subcommand(
+            "keyfinder", help="detect and add initial key from audio"
+        )
         cmd.func = self.command
         return [cmd]
 
@@ -52,28 +49,23 @@ class KeyFinderPlugin(BeetsPlugin):
         self.find_key(task.imported_items())
 
     def find_key(self, items, write=False):
-        overwrite = self.config['overwrite'].get(bool)
-        command = [self.config['bin'].as_str()]
+        overwrite = self.config["overwrite"].get(bool)
+        command = [self.config["bin"].as_str()]
         # The KeyFinder GUI program needs the -f flag before the path.
         # keyfinder-cli is similar, but just wants the path with no flag.
-        if 'keyfinder-cli' not in os.path.basename(command[0]).lower():
-            command.append('-f')
+        if "keyfinder-cli" not in os.path.basename(command[0]).lower():
+            command.append("-f")
 
         for item in items:
-            if item['initial_key'] and not overwrite:
+            if item["initial_key"] and not overwrite:
                 continue
 
             try:
-                output = util.command_output(command + [util.syspath(
-                                                        item.path)]).stdout
+                output = util.command_output(
+                    command + [util.syspath(item.path)]
+                ).stdout
             except (subprocess.CalledProcessError, OSError) as exc:
-                self._log.error(u'execution failed: {0}', exc)
-                continue
-            except UnicodeEncodeError:
-                # Workaround for Python 2 Windows bug.
-                # https://bugs.python.org/issue1759845
-                self._log.error(u'execution failed for Unicode path: {0!r}',
-                                item.path)
+                self._log.error("execution failed: {0}", exc)
                 continue
 
             try:
@@ -81,18 +73,21 @@ class KeyFinderPlugin(BeetsPlugin):
             except IndexError:
                 # Sometimes keyfinder-cli returns 0 but with no key, usually
                 # when the file is silent or corrupt, so we log and skip.
-                self._log.error(u'no key returned for path: {0}', item.path)
+                self._log.error("no key returned for path: {0}", item.path)
                 continue
 
             try:
-                key = util.text_string(key_raw)
+                key = key_raw.decode("utf-8")
             except UnicodeDecodeError:
-                self._log.error(u'output is invalid UTF-8')
+                self._log.error("output is invalid UTF-8")
                 continue
 
-            item['initial_key'] = key
-            self._log.info(u'added computed initial key {0} for {1}',
-                           key, util.displayable_path(item.path))
+            item["initial_key"] = key
+            self._log.info(
+                "added computed initial key {0} for {1}",
+                key,
+                util.displayable_path(item.path),
+            )
 
             if write:
                 item.try_write()
