@@ -3,51 +3,57 @@
 import logging as log
 import sys
 import threading
-import unittest
 from io import StringIO
 
 import beets.logging as blog
 import beetsplug
 from beets import plugins, ui
 from beets.test import _common, helper
-from beets.test._common import TestCase
+from beets.test.helper import (
+    AsIsImporterMixin,
+    BeetsTestCase,
+    ImportTestCase,
+    PluginMixin,
+)
 
 
-class LoggingTest(TestCase):
+class LoggingTest(BeetsTestCase):
     def test_logging_management(self):
         l1 = log.getLogger("foo123")
         l2 = blog.getLogger("foo123")
-        self.assertEqual(l1, l2)
-        self.assertEqual(l1.__class__, log.Logger)
+        assert l1 == l2
+        assert l1.__class__ == log.Logger
 
         l3 = blog.getLogger("bar123")
         l4 = log.getLogger("bar123")
-        self.assertEqual(l3, l4)
-        self.assertEqual(l3.__class__, blog.BeetsLogger)
-        self.assertIsInstance(
+        assert l3 == l4
+        assert l3.__class__ == blog.BeetsLogger
+        assert isinstance(
             l3, (blog.StrFormatLogger, blog.ThreadLocalLevelLogger)
         )
 
         l5 = l3.getChild("shalala")
-        self.assertEqual(l5.__class__, blog.BeetsLogger)
+        assert l5.__class__ == blog.BeetsLogger
 
         l6 = blog.getLogger()
-        self.assertNotEqual(l1, l6)
+        assert l1 != l6
 
     def test_str_format_logging(self):
-        l = blog.getLogger("baz123")
+        logger = blog.getLogger("baz123")
         stream = StringIO()
         handler = log.StreamHandler(stream)
 
-        l.addHandler(handler)
-        l.propagate = False
+        logger.addHandler(handler)
+        logger.propagate = False
 
-        l.warning("foo {0} {bar}", "oof", bar="baz")
+        logger.warning("foo {0} {bar}", "oof", bar="baz")
         handler.flush()
-        self.assertTrue(stream.getvalue(), "foo oof baz")
+        assert stream.getvalue(), "foo oof baz"
 
 
-class LoggingLevelTest(unittest.TestCase, helper.TestHelper):
+class LoggingLevelTest(AsIsImporterMixin, PluginMixin, ImportTestCase):
+    plugin = "dummy"
+
     class DummyModule:
         class DummyPlugin(plugins.BeetsPlugin):
             def __init__(self):
@@ -74,100 +80,90 @@ class LoggingLevelTest(unittest.TestCase, helper.TestHelper):
     def setUp(self):
         sys.modules["beetsplug.dummy"] = self.DummyModule
         beetsplug.dummy = self.DummyModule
-        self.setup_beets()
-        self.load_plugins("dummy")
-
-    def tearDown(self):
-        self.unload_plugins()
-        self.teardown_beets()
-        del beetsplug.dummy
-        sys.modules.pop("beetsplug.dummy")
-        self.DummyModule.DummyPlugin.listeners = None
-        self.DummyModule.DummyPlugin._raw_listeners = None
+        super().setUp()
 
     def test_command_level0(self):
         self.config["verbose"] = 0
         with helper.capture_log() as logs:
             self.run_command("dummy")
-        self.assertIn("dummy: warning cmd", logs)
-        self.assertIn("dummy: info cmd", logs)
-        self.assertNotIn("dummy: debug cmd", logs)
+        assert "dummy: warning cmd" in logs
+        assert "dummy: info cmd" in logs
+        assert "dummy: debug cmd" not in logs
 
     def test_command_level1(self):
         self.config["verbose"] = 1
         with helper.capture_log() as logs:
             self.run_command("dummy")
-        self.assertIn("dummy: warning cmd", logs)
-        self.assertIn("dummy: info cmd", logs)
-        self.assertIn("dummy: debug cmd", logs)
+        assert "dummy: warning cmd" in logs
+        assert "dummy: info cmd" in logs
+        assert "dummy: debug cmd" in logs
 
     def test_command_level2(self):
         self.config["verbose"] = 2
         with helper.capture_log() as logs:
             self.run_command("dummy")
-        self.assertIn("dummy: warning cmd", logs)
-        self.assertIn("dummy: info cmd", logs)
-        self.assertIn("dummy: debug cmd", logs)
+        assert "dummy: warning cmd" in logs
+        assert "dummy: info cmd" in logs
+        assert "dummy: debug cmd" in logs
 
     def test_listener_level0(self):
         self.config["verbose"] = 0
         with helper.capture_log() as logs:
             plugins.send("dummy_event")
-        self.assertIn("dummy: warning listener", logs)
-        self.assertNotIn("dummy: info listener", logs)
-        self.assertNotIn("dummy: debug listener", logs)
+        assert "dummy: warning listener" in logs
+        assert "dummy: info listener" not in logs
+        assert "dummy: debug listener" not in logs
 
     def test_listener_level1(self):
         self.config["verbose"] = 1
         with helper.capture_log() as logs:
             plugins.send("dummy_event")
-        self.assertIn("dummy: warning listener", logs)
-        self.assertIn("dummy: info listener", logs)
-        self.assertNotIn("dummy: debug listener", logs)
+        assert "dummy: warning listener" in logs
+        assert "dummy: info listener" in logs
+        assert "dummy: debug listener" not in logs
 
     def test_listener_level2(self):
         self.config["verbose"] = 2
         with helper.capture_log() as logs:
             plugins.send("dummy_event")
-        self.assertIn("dummy: warning listener", logs)
-        self.assertIn("dummy: info listener", logs)
-        self.assertIn("dummy: debug listener", logs)
+        assert "dummy: warning listener" in logs
+        assert "dummy: info listener" in logs
+        assert "dummy: debug listener" in logs
 
     def test_import_stage_level0(self):
         self.config["verbose"] = 0
         with helper.capture_log() as logs:
-            importer = self.create_importer()
-            importer.run()
-        self.assertIn("dummy: warning import_stage", logs)
-        self.assertNotIn("dummy: info import_stage", logs)
-        self.assertNotIn("dummy: debug import_stage", logs)
+            self.run_asis_importer()
+        assert "dummy: warning import_stage" in logs
+        assert "dummy: info import_stage" not in logs
+        assert "dummy: debug import_stage" not in logs
 
     def test_import_stage_level1(self):
         self.config["verbose"] = 1
         with helper.capture_log() as logs:
-            importer = self.create_importer()
-            importer.run()
-        self.assertIn("dummy: warning import_stage", logs)
-        self.assertIn("dummy: info import_stage", logs)
-        self.assertNotIn("dummy: debug import_stage", logs)
+            self.run_asis_importer()
+        assert "dummy: warning import_stage" in logs
+        assert "dummy: info import_stage" in logs
+        assert "dummy: debug import_stage" not in logs
 
     def test_import_stage_level2(self):
         self.config["verbose"] = 2
         with helper.capture_log() as logs:
-            importer = self.create_importer()
-            importer.run()
-        self.assertIn("dummy: warning import_stage", logs)
-        self.assertIn("dummy: info import_stage", logs)
-        self.assertIn("dummy: debug import_stage", logs)
+            self.run_asis_importer()
+        assert "dummy: warning import_stage" in logs
+        assert "dummy: info import_stage" in logs
+        assert "dummy: debug import_stage" in logs
 
 
 @_common.slow_test()
-class ConcurrentEventsTest(TestCase, helper.TestHelper):
+class ConcurrentEventsTest(AsIsImporterMixin, ImportTestCase):
     """Similar to LoggingLevelTest but lower-level and focused on multiple
     events interaction. Since this is a bit heavy we don't do it in
     LoggingLevelTest.
     """
 
+    db_on_disk = True
+
     class DummyPlugin(plugins.BeetsPlugin):
         def __init__(self, test_case):
             plugins.BeetsPlugin.__init__(self, "dummy")
@@ -186,30 +182,24 @@ class ConcurrentEventsTest(TestCase, helper.TestHelper):
 
         def listener1(self):
             try:
-                self.test_case.assertEqual(self._log.level, log.INFO)
+                assert self._log.level == log.INFO
                 self.t1_step = 1
                 self.lock1.acquire()
-                self.test_case.assertEqual(self._log.level, log.INFO)
+                assert self._log.level == log.INFO
                 self.t1_step = 2
             except Exception as e:
                 self.exc = e
 
         def listener2(self):
             try:
-                self.test_case.assertEqual(self._log.level, log.DEBUG)
+                assert self._log.level == log.DEBUG
                 self.t2_step = 1
                 self.lock2.acquire()
-                self.test_case.assertEqual(self._log.level, log.DEBUG)
+                assert self._log.level == log.DEBUG
                 self.t2_step = 2
             except Exception as e:
                 self.exc = e
 
-    def setUp(self):
-        self.setup_beets(disk=True)
-
-    def tearDown(self):
-        self.teardown_beets()
-
     def test_concurrent_events(self):
         dp = self.DummyPlugin(self)
 
@@ -220,37 +210,37 @@ class ConcurrentEventsTest(TestCase, helper.TestHelper):
         try:
             dp.lock1.acquire()
             dp.lock2.acquire()
-            self.assertEqual(dp._log.level, log.NOTSET)
+            assert dp._log.level == log.NOTSET
 
             self.config["verbose"] = 1
             t1 = threading.Thread(target=dp.listeners["dummy_event1"][0])
             t1.start()  # blocked. t1 tested its log level
             while dp.t1_step != 1:
                 check_dp_exc()
-            self.assertTrue(t1.is_alive())
-            self.assertEqual(dp._log.level, log.NOTSET)
+            assert t1.is_alive()
+            assert dp._log.level == log.NOTSET
 
             self.config["verbose"] = 2
             t2 = threading.Thread(target=dp.listeners["dummy_event2"][0])
             t2.start()  # blocked. t2 tested its log level
             while dp.t2_step != 1:
                 check_dp_exc()
-            self.assertTrue(t2.is_alive())
-            self.assertEqual(dp._log.level, log.NOTSET)
+            assert t2.is_alive()
+            assert dp._log.level == log.NOTSET
 
             dp.lock1.release()  # dummy_event1 tests its log level + finishes
             while dp.t1_step != 2:
                 check_dp_exc()
             t1.join(0.1)
-            self.assertFalse(t1.is_alive())
-            self.assertTrue(t2.is_alive())
-            self.assertEqual(dp._log.level, log.NOTSET)
+            assert not t1.is_alive()
+            assert t2.is_alive()
+            assert dp._log.level == log.NOTSET
 
             dp.lock2.release()  # dummy_event2 tests its log level + finishes
             while dp.t2_step != 2:
                 check_dp_exc()
             t2.join(0.1)
-            self.assertFalse(t2.is_alive())
+            assert not t2.is_alive()
 
         except Exception:
             print("Alive threads:", threading.enumerate())
@@ -269,28 +259,17 @@ class ConcurrentEventsTest(TestCase, helper.TestHelper):
 
         blog.getLogger("beets").set_global_level(blog.WARNING)
         with helper.capture_log() as logs:
-            importer = self.create_importer()
-            importer.run()
-        self.assertEqual(logs, [])
+            self.run_asis_importer()
+        assert logs == []
 
         blog.getLogger("beets").set_global_level(blog.INFO)
         with helper.capture_log() as logs:
-            importer = self.create_importer()
-            importer.run()
-        for l in logs:
-            self.assertIn("import", l)
-            self.assertIn("album", l)
+            self.run_asis_importer()
+        for line in logs:
+            assert "import" in line
+            assert "album" in line
 
         blog.getLogger("beets").set_global_level(blog.DEBUG)
         with helper.capture_log() as logs:
-            importer = self.create_importer()
-            importer.run()
-        self.assertIn("Sending event: database_change", logs)
-
-
-def suite():
-    return unittest.TestLoader().loadTestsFromName(__name__)
-
-
-if __name__ == "__main__":
-    unittest.main(defaultTest="suite")
+            self.run_asis_importer()
+        assert "Sending event: database_change" in logs
